'''Generate truth data from a truth_data_file.

A truth_data_file may be produced by a human being, so sometimes
it should be considered truth data. This file provides
utilities for turning a truth_data_file of a certain format into
truth data the harness understands.
'''

from __future__ import absolute_import
import argparse
import json
from bs4 import BeautifulSoup
import logging
import sys

from dossier.label import Label, LabelStore, CorefValue
import kvlayer
import yakonfig

logger = logging.getLogger(__name__)

def parse_passage(p):
    '''Extract a line_data dict from a passage's XML data and context.
    '''
    line_data = {}
    domain = p.parent.parent.parent
    topic = p.parent.parent
    subtopic = p.parent
    line_data['domain_id'] = domain['id'].encode('utf-8')
    line_data['domain_name'] = domain['name'].encode('utf-8')
    line_data['userid'] = 'dropped'
    line_data['username'] = 'dropped'
    line_data['topic_id'] = topic['id'].encode('utf-8')
    line_data['topic_name'] = topic['name'].encode('utf-8')
    line_data['subtopic_id'] = subtopic['id'].encode('utf-8')
    line_data['subtopic_name'] = subtopic['name'].encode('utf-8')
    line_data['passage_id'] = p['id'].encode('utf-8')
    line_data['passage_name'] = p.find('text').text.encode('utf-8')
    line_data['docno'] = p.docno.text.encode('utf-8')
    line_data['grade'] = p.rating.text.encode('utf-8')
    return line_data
    
def make_full_doc_id(doc_id, offset_start, offset_end):
    '''A full doc_id is of the form: doc_id#offset_start,offset_end
    '''
    offset_string = ','.join([offset_start, offset_end])
    return '#'.join([doc_id, offset_string])

def make_offset_string(offset_start, offset_end):
    '''Create an offset string from a pair of offsets.

    :param offset_start: str
    :param offset_end: str
    '''
    return ','.join([offset_start, offset_end])

def label_from_truth_data_file_line(line_data):
    '''Create a label from a *parsed* truth_data_file line.

    :param line_data: dict
    '''
    # document data
    doc_id = line_data['docno']
    if not doc_id.strip():
        logger.warn('dropping invalid truth data line: '
                    'bad docno: %r: %r'
                    % (doc_id, line_data))
        return None

    if len(line_data['passage_name'].strip()) < 1:
        logger.warn('dropping empty passage: %r', line_data)
        return None

    # annotation data
    topic_id = line_data['topic_id']
    subtopic_id = line_data['subtopic_id']
    passage_id = line_data['passage_id']
    annotator = line_data['userid']

    # value data
    value = CorefValue.Positive
    try:
        rating = int(line_data['grade'])
    except ValueError:
        logger.warn('replacing bogus grade with zero = %r',
                    line_data['grade'])
        rating = 0

    if rating < 0:
        value = CorefValue.Negative
        rating = 0

    # meta data
    meta = {'domain_name': line_data['domain_name'],
            'domain_id': line_data['domain_id'],
            'username': line_data['username'],
            'topic_name': line_data['topic_name'],
            'topic_id': line_data['topic_id'],
            'subtopic_name': line_data['subtopic_name'],
            'passage_text': line_data['passage_name']}

    label = Label(topic_id, doc_id, annotator, value,
                  subtopic_id1=subtopic_id, subtopic_id2=passage_id,
                  rating=rating, meta=meta)
    return label

def parse_truth_data(label_store, truth_data_path, batch_size=10000):
    data_file = open(truth_data_path, 'r')
    data = BeautifulSoup(data_file, 'xml')

    labels_to_put = []
    num_labels = 0
    for psg in data.find_all('passage'):
        line_data = parse_passage(psg)
        label = label_from_truth_data_file_line(line_data)
        if label is not None:
            labels_to_put.append(label)
            num_labels += 1
            if num_labels % 1000 == 0:
                logger.debug('Converted %d labels.' % num_labels)
            if len(labels_to_put) >= batch_size:
                label_store.put(*labels_to_put)
                labels_to_put = []
    if len(labels_to_put) > 0:
        label_store.put(*labels_to_put)

def main():
    parser = argparse.ArgumentParser('test tool for checking that we can load '
                                     'the truth data as distributed by NIST for '
                                     'TREC 2015')
    parser.add_argument('truth_data_path', help='path to truth data file')
    modules = [yakonfig, kvlayer]
    args = yakonfig.parse_args(parser, modules)
    logging.basicConfig(level=logging.DEBUG)
    kvl = kvlayer.client()
    label_store = LabelStore(kvl)
    parse_truth_data(label_store, args.truth_data_path)
    logger.debug('Done!  The truth data was loaded into this kvlayer backend: %r',
                 json.dumps(yakonfig.get_global_config('kvlayer'), indent=4,
                            sort_keys=True))

if __name__ == '__main__':
    main()



#!/usr/bin/env python

import os
import fnmatch
import subprocess

## prepare to run PyTest as a command
from distutils.core import Command

from setuptools import setup, find_packages

from version import get_git_version
VERSION, SOURCE_LABEL = get_git_version()
PROJECT = 'trec_dd'
URL = 'http://trec-dd.org/'
AUTHOR = 'Diffeo, Inc.'
AUTHOR_EMAIL = 'support@diffeo.com'
DESC = 'TREC Dynamic Domain (DD) evaluation test harness for simulating user interaction with a search engine'


def read_file(file_name):
    file_path = os.path.join(
        os.path.dirname(__file__),
        file_name
        )
    return open(file_path).read()


def recursive_glob(treeroot, pattern):
    results = []
    for base, dirs, files in os.walk(treeroot):
        goodfiles = fnmatch.filter(files, pattern)
        results.extend(os.path.join(base, f) for f in goodfiles)
    return results


def recursive_glob_with_tree(treeroot, pattern):
    results = []
    for base, dirs, files in os.walk(treeroot):
        goodfiles = fnmatch.filter(files, pattern)
        one_dir_results = []
        for f in goodfiles:
            one_dir_results.append(os.path.join(base, f))
        results.append((base, one_dir_results))
    return results


def _myinstall(pkgspec):
    subprocess.check_call(['pip', 'install', pkgspec])


class PyTest(Command):
    '''run py.test'''

    description = 'runs py.test to execute all tests'

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if self.distribution.install_requires:
            for ir in self.distribution.install_requires:
                _myinstall(ir)
        if self.distribution.tests_require:
            for ir in self.distribution.tests_require:
                _myinstall(ir)

        errno = subprocess.call(['py.test', '-n', '3', '-s', 'trec_dd', '--runslow', '--runperf'])
        raise SystemExit(errno)

setup(
    name=PROJECT,
    version=VERSION,
    description=DESC,
    long_description=read_file('README.rst'),
    author=AUTHOR,
    license='MIT/X11 license http://opensource.org/licenses/MIT',
    author_email=AUTHOR_EMAIL,
    url=URL,
    packages=find_packages(),
    cmdclass={
        'test': PyTest,
    },
    # We can select proper classifiers later
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',  ## MIT/X11 license http://opensource.org/licenses/MIT
    ],
    install_requires=[
        'dossier.label',
        'lxml',
        'beautifulsoup4',
    ],
    extras_require={
        'postgres': [
            'psycopg2',
        ],
        'mysql': [
            'kvlayer_mysql',
        ],
    },        
    entry_points={
        'console_scripts': [
            'trec_dd_harness = trec_dd.harness.run:main',
            'trec_dd_scorer = trec_dd.scorer.run:main',
            'trec_dd_random_system = trec_dd.system.random_system:main',
        ]
    },
    scripts=['bin/cubeTest.pl'],
    include_package_data=True,
)
